
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JProgressBar;

public class ProgressFrame extends JDialog implements ActionListener {

  JLabel progressFromFile = new JLabel();
  JLabel progressToFile = new JLabel();
  JLabel progressReceived = new JLabel();
  JLabel progressStatus = new JLabel();
  JLabel progressConnectionName = new JLabel();
  JButton cancelButton = new JButton( "Cancel" );
  JButton doneButton = new JButton( "Done" );
  JProgressBar progressBar;

  public ProgressFrame( java.awt.Frame parent ) {
    super( parent );

    progressBar = new JProgressBar( 0, 1 );
    progressBar.setStringPainted( true );
    progressBar.setPreferredSize( new Dimension( 400, 20 ) );
    cancelButton.setOpaque( false );
    doneButton.setOpaque( false );
    doneButton.setEnabled( false );
    doneButton.addActionListener( this );

    getContentPane().setLayout( new GridBagLayout() );
    GridBagConstraints gbc = new GridBagConstraints();

    gbc.insets = new Insets( 3, 3, 3, 3 );
    gbc.fill = gbc.NONE;
    gbc.anchor = gbc.EAST;
    gbc.gridx = 0;
    gbc.gridy = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    getContentPane().add( new JLabel( "Connection:" ), gbc );

    gbc.gridy += gbc.gridheight;
    getContentPane().add( new JLabel( "From:" ), gbc );

    gbc.gridy += gbc.gridheight;
    getContentPane().add( new JLabel( "To:" ), gbc );

    gbc.gridy += gbc.gridheight;
    getContentPane().add( new JLabel( "Received:" ), gbc );

    gbc.gridy += gbc.gridheight;
    getContentPane().add( new JLabel( "Progress:" ), gbc );

    gbc.gridy += gbc.gridheight;
    getContentPane().add( new JLabel( "Status:" ), gbc );

    gbc.fill = gbc.HORIZONTAL;
    gbc.anchor = gbc.WEST;
    gbc.gridy = 0;
    gbc.gridx += gbc.gridwidth;
    getContentPane().add( progressConnectionName, gbc );

    gbc.gridy += gbc.gridheight;
    getContentPane().add( progressFromFile, gbc );

    gbc.gridy += gbc.gridheight;
    getContentPane().add( progressToFile, gbc );

    gbc.gridy += gbc.gridheight;
    getContentPane().add( progressReceived, gbc );

    gbc.gridy += gbc.gridheight;
    getContentPane().add( progressBar, gbc );

    gbc.gridy += gbc.gridheight;
    getContentPane().add( progressStatus, gbc );

    gbc.fill = gbc.NONE;
    gbc.anchor = gbc.WEST;
    gbc.gridx = 0;
    gbc.gridy += gbc.gridheight;
    getContentPane().add( cancelButton, gbc );

    gbc.anchor = gbc.EAST;
    gbc.gridx += gbc.gridwidth;
    getContentPane().add( doneButton, gbc );

    pack();
  }

  public void actionPerformed( ActionEvent ev ) {
    // gotta be the done button;
    if( ev.getSource() == doneButton ) setVisible( false );
  }

  public JButton getCancelButton() {
    return cancelButton;
  }

  public void setValue( int value ) {
    if( value > progressBar.getMaximum() ) {
      progressBar.getModel().setValue( progressBar.getMaximum() );
    } else {
      progressBar.getModel().setValue( value );
    }
    setProgressReceived();
  }

  public int getValue() {
    return progressBar.getModel().getValue();
  }

  public void addValue( int value ) {
    setValue( getValue() + value );
  }

  public void setStatus( String status ) {
    if( status.length() > 80 ) status = "<html><small>"+status+"</small>";
    progressStatus.setText( status );
  }

  public void setCompleted() {
    doneButton.setEnabled( true );
    cancelButton.setEnabled( false );
  }

  public void setMaximum( int value ) {
    progressBar.setMaximum( value );
    setProgressReceived();
  }

  public int getMaximum() {
    return progressBar.getMaximum();
  }

  public void setConnectionName( String name ) {
    progressConnectionName.setText( name );
  }

  public void setFromFile( String name ) {
    progressFromFile.setText( name );
  }

  public void setToFile( String name ) {
    progressToFile.setText( name );
  }

  private void setProgressReceived() {
    String s = "" + progressBar.getValue();
    if( progressBar.getMaximum() != 1 ) s += " of " + progressBar.getMaximum();
    progressReceived.setText( s );
  }
}
